// --------------------------------------------------------
// Disc Practice mission type
// --------------------------------------------------------

// DisplayName = Disc Practice

//--- GAME RULES BEGIN ---
//Disc launcher, Energy Pack and light armor only
//Designed to concentrate on disc shots
//Gametype written by SPINEBUSTER
//--- GAME RULES END ---

$DP::Version = 1.0;
$DP::VersionString = "1.0";

// ========================================================================== //
// |                                                                        | //
// |  PREFERENCES                                                           | //
// |                                                                        | //
// ========================================================================== //

// Attempt to load DPPrefs file
$DP::PrefsLoaded = 0;
exec( "prefs/DPPrefs.cs" );
  
if ( $DP::PrefsLoaded != $DP::Version )
{
  // DPPrefs not loaded or invalid
  
  if ( $DP::PrefsLoaded == 0 )
    echo( "prefs/DPPrefs.cs not found - writing new file" );
  else
  {
    // There is a prefs file but it's an old version
    // Make a backup of it before writing out a new one
    
    echo( "prefs/DPPrefs.cs is wrong version ("@$DP::PrefsLoaded@") - writing new file" );
      
    if ( tsCopyTextFile( "prefs/DPPrefs.cs", "prefs/DPPrefs.old" ) )
      echo( "backup saved to prefs/DPPrefs.old" );
  }
  
  // Create new DPPrefs.cs file from arenaPrefs.default file included in distribution
  
  if ( tsCopyTextFile( "scripts/DPPrefs.default", "prefs/DPPrefs.cs" ) )
    exec( "prefs/DPPrefs.cs" );
  else
  {
    // Failed to write new DPPrefs file - use internal default prefs for now
    
    echo( "ERROR! COULD NOT WRITE NEW DPPREFS FILE - using builtin defaults" );
    
    $DP::Pref::RestrictArmor = 2;

    $DP::PrefsLoaded = 0.1;
  }
}

// ========================================================================== //
// |                                                                        | //
// |  INITIALIZATION                                                        | //
// |                                                                        | //
// ========================================================================== //

// ------------------------------------------------------------------ //
// DPGame::activatePackages()
// Activate all required overrides
// DP uses this as a general low-level initialization point

function DPGame::activatePackages( %game )
{
  // Activate DefaultGame
  //  IMPLEMENTATION: The server code is supposed to activate DefaultGame
  //    but it doesn't do so if this function ("activatePackages") is
  //    overridden. This is very strange. As a result we just do it ourselves.
  activatePackage( DefaultGame );
  
  // Activate DP packages
  activatePackage( DPInventory );
  
  // Need to initialize this here since it's used before initGameVars() is called
  // Check initGameVars() for more explanation
  %game.state = "countdown";
}

// ------------------------------------------------------------------ //
// DPGame::allowsProtectedStatics()
// Option Call from DefaultGame - Prevents base assets from being destroyed

function DPGame::allowsProtectedStatics( %game )
{
  return true;
}


// ========================================================================== //
// |                                                                        | //
// |  SHUTDOWN                                                              | //
// |                                                                        | //
// ========================================================================== //

// ------------------------------------------------------------------ //
// DPGame::deactivatePackages()
// Deactivate all required overrides
// DP uses this as a general low-level shutdown point

function DPGame::deactivatePackages( %game )
{
  // Deactivate DP packages
  //  Note that packages are being deactivated in the reverse of the order
  //  they were activated in
  deactivatePackage( DPInventory );
  
  // Deactivate DefaultGame
  deactivatePackage( DefaultGame );
}


function DPGame::setUpTeams(%game)
{  
   %group = nameToID("MissionGroup/Teams");
   if(%group == -1)
      return;
   
   // create a team0 if it does not exist
   %team = nameToID("MissionGroup/Teams/team0");
   if(%team == -1)
   {
      %team = new SimGroup("team0");
      %group.add(%team);
   }

   // 'team0' is not counted as a team here
   %game.numTeams = 0;
   while(%team != -1)
   {
      // create drop set and add all spawnsphere objects into it
      %dropSet = new SimSet("TeamDrops" @ %game.numTeams);
      MissionCleanup.add(%dropSet);

      %spawns = nameToID("MissionGroup/Teams/team" @ %game.numTeams @ "/SpawnSpheres");
      if(%spawns != -1)
      {
         %count = %spawns.getCount();
         for(%i = 0; %i < %count; %i++)
            %dropSet.add(%spawns.getObject(%i));
      }

      // set the 'team' field for all the objects in this team
      %team.setTeam(0);

      clearVehicleCount(%team+1);
      // get next group
      %team = nameToID("MissionGroup/Teams/team" @ %game.numTeams + 1);
      if (%team != -1)
         %game.numTeams++;
   }

   // set the number of sensor groups (including team0) that are processed
   setSensorGroupCount(%game.numTeams + 1);
   %game.numTeams = 1;

   // allow teams 1->31 to listen to each other (team 0 can only listen to self)
   for(%i = 1; %i < 32; %i++)
      setSensorGroupListenMask(%i, 0xfffffffe);
}

function DPGame::initGameVars(%game)
{
   %game.SCORE_PER_KILL = 1; 
   %game.SCORE_PER_DEATH = -1;
   %game.SCORE_PER_SUICIDE = -1;
}  


// ========================================================================== //
// |                                                                        | //
// |  INVENTORY                                                             | //
// |                                                                        | //
// ========================================================================== //

$InvBanList[DP, "TurretOutdoorDeployable"] = 1;
$InvBanList[DP, "TurretIndoorDeployable"] = 1;
$InvBanList[DP, "ElfBarrelPack"] = 1;
$InvBanList[DP, "MortarBarrelPack"] = 1;
$InvBanList[DP, "PlasmaBarrelPack"] = 1;
$InvBanList[DP, "AABarrelPack"] = 1;
$InvBanList[DP, "MissileBarrelPack"] = 1;
$InvBanList[DP, "Mine"] = 1;
$InvBanList[DP, "ShieldPack"] = 1;
$InvBanList[DP, "CloakPack"] = 1;
$InvBanList[DP, "SensorJammerPack"] = 1;
$InvBanList[DP, "MotionSensorDeployable"] = 1;
$InvBanList[DP, "PulseSensorDeployable"] = 1;
$InvBanList[DP, "AmmoPack"] = 1;
$InvBanList[DP, "SatchelCharge"] = 1;
$InvBanList[DP, "RepairPack"] = 1;
$InvBanList[DP, "InventoryDeployable"] = 1;
$InvBanList[DP, "FlashGrenade"] = 1;
$InvBanList[DP, "ConcussionGrenade"] = 1;
$InvBanList[DP, "FlareGrenade"] = 1;
$InvBanList[DP, "CameraGrenade"] = 1;
$InvBanList[DP, "Grenade"] = 1;
$InvBanList[DP, "ELFGun"] = 1;
$InvBanList[DP, "ShockLance"] = 1;
$InvBanList[DP, "Chaingun"] = 1;
$InvBanList[DP, "MissileLauncher"] = 1;
$InvBanList[DP, "Mortar"] = 1;
$InvBanList[DP, "GrenadeLauncher"] = 1;
$InvBanList[DP, "SniperRifle"] = 1;
$InvBanList[DP, "Plasma"] = 1;
$InvBanList[DP, "Blaster"] = 1;

function DPGame::equip(%game, %player)
{
   for(%i =0; %i<$InventoryHudCount; %i++)
      %player.client.setInventoryHudItem($InventoryHudData[%i, itemDataName], 0, 1);
   %player.client.clearBackpackIcon();
	buyFavorites(%player.client);	
	%player.cycleWeapon("next");	
}                  

// ------------------------------------------------------------------ //
// BEGIN PACKAGE [ DPInventory ]
// ------------------------------------------------------------------ //

package DPInventory
{

// ------------------------------------------------------------------ //
// serverCmdSetClientFav()
// Called when the client tries to set it's buy list

function serverCmdSetClientFav( %client, %text )
{
  // Disallow certain armors

  if ( $DP::Pref::RestrictArmor == 2 )
    %text = setField( %text, 1, "Scout" );
  else if ( $DP::Pref::RestrictArmor == 1 )
    if ( getField( %text, 1 ) $= "Juggernaut" )
      %text = setField( %text, 1, "Assault" );

  Parent::serverCmdSetClientFav( %client, %text );
}

};

// ------------------------------------------------------------------ //
// END PACKAGE [ DPInventory ]
// ------------------------------------------------------------------ //


function DPGame::pickPlayerSpawn(%game, %client, %respawn)
{
   // all spawns come from team 1
   return %game.pickTeamSpawn(1);
}

function DPGame::clientJoinTeam( %game, %client, %team, %respawn )
{
   %game.assignClientTeam( %client );
   
   // Spawn the player:
   %game.spawnPlayer( %client, %respawn );
}

function DPGame::assignClientTeam(%game, %client)
{
   for(%i = 1; %i < 32; %i++)
      $DPTeamArray[%i] = false;

   %maxSensorGroup = 0;
   %count = ClientGroup.getCount();
   for(%i = 0; %i < %count; %i++)
   {
      %cl = ClientGroup.getObject(%i);
      if(%cl != %client)
      {
         $DPTeamArray[%cl.team] = true;
         if (%cl.team > %maxSensorGroup)
            %maxSensorGroup = %cl.team;
      }
   }

   //now loop through the team array, looking for an empty team
   for(%i = 1; %i < 32; %i++)
   {
      if (! $DPTeamArray[%i])
      {
         %client.team = %i;
         if (%client.team > %maxSensorGroup)
            %maxSensorGroup = %client.team;
         break;
      }
   }

   // set player's skin pref here
   setTargetSkin(%client.target, %client.skin);

   // Let everybody know you are no longer an observer:
   messageAll( 'MsgClientJoinTeam', '\c1%1 has joined the fray.', %client.name, "", %client, 1 );
   updateCanListenState( %client );

   //now set the max number of sensor groups...
   setSensorGroupCount(%maxSensorGroup + 1);
}

function DPGame::clientMissionDropReady(%game, %client)
{
   messageClient(%client, 'MsgClientReady',"", %game.class);
   messageClient(%client, 'MsgYourScoreIs', "", 0);
   messageClient(%client, 'MsgDPPlayerDies', "", 0);
   messageClient(%client, 'MsgDPKill', "", 0);
   %game.resetScore(%client);

   messageClient(%client, 'MsgMissionDropInfo', '\c0You are in mission %1 (%2).', $MissionDisplayName, $MissionTypeDisplayName, $ServerName ); 
   
   DefaultGame::clientMissionDropReady(%game, %client);
}

function DPGame::AIHasJoined(%game, %client)
{
   //let everyone know the player has joined the game
   //messageAllExcept(%client, -1, 'MsgClientJoinTeam', '%1 has joined the fray.', %client.name, "", %client, 1 );
}

function DPGame::checkScoreLimit(%game, %client)
{
   //there's no score limit in DP
}

function DPGame::createPlayer(%game, %client, %spawnLoc, %respawn)
{
   DefaultGame::createPlayer(%game, %client, %spawnLoc, %respawn);
   %client.setSensorGroup(%client.team);
}

function DPGame::resetScore(%game, %client)
{
   %client.deaths = 0;
   %client.kills = 0;
   %client.score = 0;
   %client.efficiency = 0.0;
   %client.suicides = 0;
}

function DPGame::onClientKilled(%game, %clVictim, %clKiller, %damageType, %implement, %damageLoc)
{
   cancel(%clVictim.player.alertThread);
   DefaultGame::onClientKilled(%game, %clVictim, %clKiller, %damageType, %implement, %damageLoc);
}

function DPGame::updateKillScores(%game, %clVictim, %clKiller, %damageType, %implement)
{
   if (%game.testKill(%clVictim, %clKiller)) //verify victim was an enemy
   {
      %game.awardScoreKill(%clKiller);
      messageClient(%clKiller, 'MsgDPKill', "", %clKiller.kills);
      %game.awardScoreDeath(%clVictim);
   }       
   else if (%game.testSuicide(%clVictim, %clKiller, %damageType))  //otherwise test for suicide
      %game.awardScoreSuicide(%clVictim);     

   messageClient(%clVictim, 'MsgDPPlayerDies', "", %clVictim.deaths + %clVictim.suicides);
}

function DPGame::recalcScore(%game, %client)
{
   %killValue = %client.kills * %game.SCORE_PER_KILL;
   %deathValue = %client.deaths * %game.SCORE_PER_DEATH;
   %suicideValue = %client.suicides * %game.SCORE_PER_SUICIDE;

   if (%killValue - %deathValue == 0)
      %client.efficiency = %suicideValue;
   else
      %client.efficiency = ((%killValue * %killValue) / (%killValue - %deathValue)) + %suicideValue;

   %client.score = mFloatLength(%client.efficiency, 1);
   messageClient(%client, 'MsgYourScoreIs', "", %client.score);
   %game.recalcTeamRanks(%client);
   %game.checkScoreLimit(%client);
}

function DPGame::timeLimitReached(%game)
{
   logEcho("game over (timelimit)");
   %game.gameOver();
   cycleMissions();
}

function DPGame::scoreLimitReached(%game)
{
   logEcho("game over (scorelimit)");
   %game.gameOver();
   cycleMissions();
}

function DPGame::gameOver(%game)
{
   //call the default
   DefaultGame::gameOver(%game);

   messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.gameover.wav" );

   cancel(%game.timeThread);
   messageAll('MsgClearObjHud', "");
   for(%i = 0; %i < ClientGroup.getCount(); %i ++) {
      %client = ClientGroup.getObject(%i);
      %game.resetScore(%client);
   }
}

function DPGame::enterMissionArea(%game, %playerData, %player)
{
   %player.client.outOfBounds = false; 
   messageClient(%player.client, 'EnterMissionArea', '\c1You are back in the mission area.');
   logEcho(%player.client.nameBase@" (pl "@%player@"/cl "@%player.client@") entered mission area");
   cancel(%player.alertThread);
}

function DPGame::leaveMissionArea(%game, %playerData, %player)
{
   if(%player.getState() $= "Dead")
      return;
                                         
   %player.client.outOfBounds = true;
   messageClient(%player.client, 'LeaveMissionArea', '\c1You have left the mission area. Return or take damage.~wfx/misc/warning_beep.wav');
   logEcho(%player.client.nameBase@" (pl "@%player@"/cl "@%player.client@") left mission area");
   %player.alertThread = %game.schedule(1000, "DPAlertPlayer", 3, %player);
}

function DPGame::DPAlertPlayer(%game, %count, %player)
{
   // MES - I commented below line out because it prints a blank line to chat window
   //messageClient(%player.client, 'MsgDPLeftMisAreaWarn', '~wfx/misc/red_alert.wav');
   if(%count > 1)
      %player.alertThread = %game.schedule(1000, "DPAlertPlayer", %count - 1, %player);
   else 
      %player.alertThread = %game.schedule(1000, "MissionAreaDamage", %player);
}

function DPGame::MissionAreaDamage(%game, %player)
{
   if(%player.getState() !$= "Dead") {                                   
      %player.setDamageFlash(0.1);
      %prevHurt = %player.getDamageLevel();
      %player.setDamageLevel(%prevHurt + 0.05);
      %player.alertThread = %game.schedule(1000, "MissionAreaDamage", %player);
   }
   else
      %game.onClientKilled(%player.client, 0, $DamageType::OutOfBounds);
}

function DPGame::updateScoreHud(%game, %client, %tag)
{
   // Clear the header:
   messageClient( %client, 'SetScoreHudHeader', "", "" );

   // Send the subheader:
   messageClient(%client, 'SetScoreHudSubheader', "", '<tab:15,235,340,415>\tPLAYER\tRATING\tKILLS\tDEATHS');

   for (%index = 0; %index < $TeamRank[0, count]; %index++)
   {
      //get the client info
      %cl = $TeamRank[0, %index];

      //get the score
      %clScore = mFloatLength( %cl.efficiency, 1 );

      %clKills = mFloatLength( %cl.kills, 0 );
      %clDeaths = mFloatLength( %cl.deaths + %cl.suicides, 0 );
      %clStyle = %cl == %client ? "<color:dcdcdc>" : "";

      //if the client is not an observer, send the message
      if (%client.team != 0)
      {
         messageClient( %client, 'SetLineHud', "", %tag, %index, '%5<tab:20, 450>\t<clip:200>%1</clip><rmargin:280><just:right>%2<rmargin:370><just:right>%3<rmargin:460><just:right>%4', 
               %cl.name, %clScore, %clKills, %clDeaths, %clStyle );
      }
      //else for observers, create an anchor around the player name so they can be observed
      else
      {
         messageClient( %client, 'SetLineHud', "", %tag, %index, '%5<tab:20, 450>\t<clip:200><a:gamelink\t%6>%1</a></clip><rmargin:280><just:right>%2<rmargin:370><just:right>%3<rmargin:460><just:right>%4', 
               %cl.name, %clScore, %clKills, %clDeaths, %clStyle, %cl );
      }
   }

   // Tack on the list of observers:
   %observerCount = 0;
   for (%i = 0; %i < ClientGroup.getCount(); %i++)
   {
      %cl = ClientGroup.getObject(%i);
      if (%cl.team == 0)
         %observerCount++;
   }

   if (%observerCount > 0)
   {
	   messageClient( %client, 'SetLineHud', "", %tag, %index, "");
      %index++;
		messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:10, 310><spush><font:Univers Condensed:22>\tOBSERVERS (%1)<rmargin:260><just:right>TIME<spop>', %observerCount);
      %index++;
      for (%i = 0; %i < ClientGroup.getCount(); %i++)
      {
         %cl = ClientGroup.getObject(%i);
         //if this is an observer
         if (%cl.team == 0)
         {
            %obsTime = getSimTime() - %cl.observerStartTime;
            %obsTimeStr = %game.formatTime(%obsTime, false);
		      messageClient( %client, 'SetLineHud', "", %tag, %index, '<tab:20, 310>\t<clip:150>%1</clip><rmargin:260><just:right>%2',
		                     %cl.name, %obsTimeStr );
            %index++;
         }
      }
   }

   //clear the rest of Hud so we don't get old lines hanging around...
   messageClient( %client, 'ClearHud', "", %tag, %index );
}

function DPGame::applyConcussion(%game, %player)
{
}
